/**
  * File: GroupHandlerFactoryImpl.java
  * Content: Implementation of the GroupHandlerFactory
  * Author: LuisM Pena
  * Date: 3nd December 2000
  * Version: 0.29.00
  * Last changes:
  *
  **/

package sensei.middleware.gmns;

import sensei.gms.ActiveGroupMember;
import sensei.middleware.gms.GroupMember;
import sensei.middleware.gms.GroupHandler;

/**
  * Implementation of the GroupHandlerFactory
  **/
public class GroupHandlerFactoryImpl extends GroupHandlerFactoryPOA
{
  public GroupHandlerFactoryImpl(GroupMember member) throws Exception
  {
    this.member = member;
    id = sensei.middleware.util.ORBcentral.getPOA().activate_object(this);
    thisGroupHandlerFactory = GroupHandlerFactoryHelper.narrow
        (sensei.middleware.util.ORBcentral.getPOA().id_to_reference(id));
  }

  public void deactivate() throws Exception
  {
    synchronized (this)
    {
      if (thisGroupHandlerFactory!=null)
        sensei.middleware.util.ORBcentral.getPOA().deactivate_object(id);
      thisGroupHandlerFactory=null;
    }
  }

  public final GroupHandlerFactory theGroupHandlerFactory()
  {
    return thisGroupHandlerFactory;
  }

  /**
    * Creates a new handler. Returns null if no handle can be created
    **/
  public GroupHandler create() throws GroupHandlerFactoryException
  {
    GroupHandler ret = null;
    try
    {
      ret = new ActiveGroupMember(member).theSenseiGMSMember();
    }
    catch (Exception ex)
    {
      ret=null;
      throw new GroupHandlerFactoryException(ex.getMessage());
    }
    return ret;
  }

  GroupMember member;
  protected GroupHandlerFactory thisGroupHandlerFactory;
  byte[] id;
}
