/**
  * File: MonitorFactory.java
  * Content: Class to send Monitor messages.
  * Author: LuisM Pena
  * Date: 03rd December 2000
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.domains;

import sensei.middleware.domains.DomainGroupHandler;
import sensei.middleware.domains.MemberStateException;
import sensei.middleware.domains.MonitorMessage;
import sensei.middleware.domains.MonitorState;

import sensei.util.Debug;
/**
  * Class to send Monitor messages.
  **/
class MonitorFactory
{

//*************************************************************************************//
//**************************** CONSTRUCTOR ********************************************//
//*************************************************************************************//

  /**
    * It is required the groupHandler to use to cast the messages, and the subgroupId
    * assigned the monitor.
    **/
  public MonitorFactory(int subgroupId, DomainGroupHandler group)
  {
    Trace.code("MonitorFactory.java -> MonitorFactory ( int subgroupId , DomainGroupHandler group )");
    this.group=group;
    this.subgroupId = subgroupId;
  }

  public void freeResources()
  {
    Trace.code("MonitorFactory.java -> void freeResources ( )");
    group=null;
  }

  public void setSubgroup(int subgroupId)
  {
    Trace.code("MonitorFactory.java -> void setSubgroup ( int subgroupId )");
    this.subgroupId = subgroupId;
  }

  public void castLock() throws MemberStateException, Exception
  {
    Trace.code("MonitorFactory.java -> void castLock ( )");
    Debug.assert(group!=null, Consts.AREA,"MonitorFactory::castLock");
    group.syncCastDomainMessage(new MonitorMessageImpl(subgroupId, true), true);
  }

  public void castUnlock() throws MemberStateException, Exception
  {
    Trace.code("MonitorFactory.java -> void castUnlock ( )");
    Debug.assert(group!=null, Consts.AREA,"MonitorFactory::castUnock");
    group.syncCastDomainMessage(new MonitorMessageImpl(subgroupId, false), true);
  }

  public static MonitorState createState(int[] locks)
  {
    Trace.code("MonitorFactory.java -> MonitorState createState ( int [ ] locks )");
    return new MonitorStateImpl(locks);
  }

//*************************************************************************************//
//**************************** INNER CLASSES *******************************************//
//*************************************************************************************//

  static class MonitorMessageImpl extends MonitorMessage
  {
    public MonitorMessageImpl (int subgroupId, boolean lock)
    {
      this.subgroup=subgroupId;
      this.lock=lock;
      this.unqueuedOnST=true; //messages are received during state transfers
      unTransactionable=lock; //lock messages are not queued on transactions!
    }
  }

  static class MonitorStateImpl extends MonitorState
  {
    public MonitorStateImpl (int[] locks) {this.locks=locks;}
  }

//*************************************************************************************//
//**************************** DATA MEMBERS *******************************************//
//*************************************************************************************//

  DomainGroupHandler group;
  int subgroupId;
}
