/**
  * File: STGroupMemberBasicStateHandler.java
  * Content: StateTransferGroupMember with BasicStateHandler behaviour
  * Author: LuisM Pena
  * Date: 31st Octonber 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.domains;

import sensei.middleware.gms.GroupHandler;
import sensei.middleware.gms.GroupMember;
import sensei.middleware.gms.Message;
import sensei.middleware.gms.View;
import sensei.middleware.domains.PhaseCoordination;
import sensei.middleware.domains.PhaseCoordinationHolder;
import sensei.middleware.domains.State;
import sensei.middleware.domains.BasicStateHandler;
import sensei.middleware.domains.StateTransferType;

import sensei.util.Debug;

/**
  * StateTransferGroupMember without state transfer
  **/
class STGroupMemberBasicStateHandler extends STGroupMember
{

  //*************************************************************************************//
  //**************************** CONSTRUCTOR ********************************************//
  //*************************************************************************************//

  public STGroupMemberBasicStateHandler(BasicStateHandler groupMember)
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> STGroupMemberBasicStateHandler ( BasicStateHandler groupMember )");
    this.groupMember = groupMember;
  }

  public GroupMember getGroupMember(){return groupMember;}
  public boolean needsTransfer(){return true;}
  public StateTransferType getStateTransferType(){return StateTransferType.STT_BasicStateHandler;}
  public void freeResources(){groupMember=null;}

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public void processPTPMessage(int sender, Message msg) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void processPTPMessage ( int sender , Message msg )");
    groupMember.processPTPMessage(sender, msg);
  }
  public void processCastMessage(int sender, Message msg) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void processCastMessage ( int sender , Message msg )");
    groupMember.processCastMessage(sender, msg);
  }
  public void memberAccepted(int identity, GroupHandler handler, View theView) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void memberAccepted ( int identity , GroupHandler handler , View theView )");
    groupMember.memberAccepted(identity, handler, theView);
  }
  public void changingView() throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void changingView ( )");
    groupMember.changingView();
  }
  public void installView(View theView) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void installView ( View theView )");
    if (sendStop)
    {
      //there was an interrupted call not followed by a stop one
      groupMember.stopTransfer(excludedMembers, false);
      sendStop=false;
    }
    groupMember.installView(theView);
  }
  public void excludedFromGroup() throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void excludedFromGroup ( )");
    groupMember.excludedFromGroup();
  }

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public void assumeState() throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void assumeState ( )");
    groupMember.assumeState();
  }
  public void startTransfer(int[] joiningMembers, PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void startTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder phase )");
    groupMember.startTransfer(joiningMembers, phase);
  }
  public State getState(PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> State getState ( PhaseCoordinationHolder phase )");
    return groupMember.getState(phase);
  }
  public void setState(State s, PhaseCoordination phase) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void setState ( State s , PhaseCoordination phase )");
    groupMember.setState(s, phase);
  }
  public void stopTransfer(int[] joiningMembers, boolean transferFinished) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void stopTransfer ( int [ ] joiningMembers , boolean transferFinished )");
    sendStop=false;
    groupMember.stopTransfer(joiningMembers, transferFinished);
  }
  public void syncTransfer(int coordinator, PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void syncTransfer ( int coordinator , PhaseCoordinationHolder phase )");
    phase.value=null;
  }
  public void interruptTransfer(PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void interruptTransfer ( PhaseCoordinationHolder phase )");
    sendStop=true;
  }
  public void continueTransfer(int[] joiningMembers, PhaseCoordinationHolder coordinator,
                                        PhaseCoordination joining) throws Exception
  {
    Trace.code("STGroupMemberBasicStateHandler.java -> void continueTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder coordinator , PhaseCoordination joining )");
    coordinator.value=null;
    groupMember.startTransfer(joiningMembers, coordinator);
  }

  //*************************************************************************************//
  //**************************** DATA MEMBERS *******************************************//
  //*************************************************************************************//

  boolean sendStop;
  BasicStateHandler groupMember;
  static int excludedMembers[]=new int[0];
}

