/**
  * File: STGroupMemberCheckpointable.java
  * Content: StateTransferGroupMember with Checkpointable or extended Checkpointable behaviour
  * Author: LuisM Pena
  * Date: 31st Octonber 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.domains;

import sensei.middleware.gms.GroupHandler;
import sensei.middleware.gms.GroupMember;
import sensei.middleware.gms.Message;
import sensei.middleware.gms.View;
import sensei.middleware.domains.DomainsNarrower;
import sensei.middleware.domains.PhaseCoordination;
import sensei.middleware.domains.PhaseCoordinationHolder;
import sensei.middleware.domains.State;
import sensei.middleware.domains.Checkpointable;
import sensei.middleware.domains.ExtendedCheckpointable;
import sensei.middleware.domains.StateTransferType;

import sensei.util.Debug;

/**
  * StateTransferGroupMember with Checkpointable or extended Checkpointable behaviour
  **/
class STGroupMemberCheckpointable extends STGroupMember
{

  //*************************************************************************************//
  //**************************** CONSTRUCTOR ********************************************//
  //*************************************************************************************//

  public STGroupMemberCheckpointable(Checkpointable groupMember)
  {
    Trace.code("STGroupMemberCheckpointable.java -> STGroupMemberCheckpointable ( Checkpointable groupMember )");
    this.groupMember = groupMember;
    this.extendedGroupMember = DomainsNarrower.toExtendedCheckpointable(groupMember);
    stateTransferType = extendedGroupMember==null?
      StateTransferType.STT_Checkpointable : StateTransferType.STT_ExtendedCheckpointable;
  }

  public GroupMember getGroupMember(){return groupMember;}
  public boolean needsTransfer(){return true;}
  public StateTransferType getStateTransferType(){return stateTransferType;}
  public void freeResources(){groupMember=null;}

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public void processPTPMessage(int sender, Message msg) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void processPTPMessage ( int sender , Message msg )");
    groupMember.processPTPMessage(sender, msg);
  }
  public void processCastMessage(int sender, Message msg) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void processCastMessage ( int sender , Message msg )");
    groupMember.processCastMessage(sender, msg);
  }
  public void memberAccepted(int identity, GroupHandler handler, View theView) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void memberAccepted ( int identity , GroupHandler handler , View theView )");
    groupMember.memberAccepted(identity, handler, theView);
  }
  public void changingView() throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void changingView ( )");
    groupMember.changingView();
  }
  public void installView(View theView) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void installView ( View theView )");
    groupMember.installView(theView);
  }
  public void excludedFromGroup() throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void excludedFromGroup ( )");
    groupMember.excludedFromGroup();
  }

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public void assumeState() throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void assumeState ( )");
    if (extendedGroupMember!=null)
      extendedGroupMember.assumeState();
  }
  public void startTransfer(int[] joiningMembers, PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void startTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder phase )");
    phase.value=startPhase;
  }
  public State getState(PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> State getState ( PhaseCoordinationHolder phase )");
    phase.value = endedPhase;
    return groupMember.getState();
  }
  public void setState(State s, PhaseCoordination phase) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void setState ( State s , PhaseCoordination phase )");
    Debug.assert(phase.transferFinished, Consts.AREA,"STGroupMemberCheckpointable::setState::1");
    groupMember.setState(s);
  }
  public void stopTransfer(int[] joiningMembers, boolean transferFinished) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void stopTransfer ( int [ ] joiningMembers , boolean transferFinished )");
  }
  public void syncTransfer(int coordinator, PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void syncTransfer ( int coordinator , PhaseCoordinationHolder phase )");
    phase.value=startPhase;
  }
  public void interruptTransfer(PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void interruptTransfer ( PhaseCoordinationHolder phase )");
    phase.value=startPhase;
  }
  public void continueTransfer(int[] joiningMembers, PhaseCoordinationHolder coordinator,
                                        PhaseCoordination joining) throws Exception
  {
    Trace.code("STGroupMemberCheckpointable.java -> void continueTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder coordinator , PhaseCoordination joining )");
    coordinator.value=startPhase;
  }

  //*************************************************************************************//
  //**************************** DATA MEMBERS *******************************************//
  //*************************************************************************************//

  Checkpointable groupMember;
  ExtendedCheckpointable extendedGroupMember;
  StateTransferType stateTransferType;
  static PhaseCoordination endedPhase = new PhaseCoordinationImpl(true);
  static PhaseCoordination startPhase = new PhaseCoordinationImpl(false);
}

