/**
  * File: STGroupMemberStateless.java
  * Content: StateTransferGroupMember without state transfer
  * Author: LuisM Pena
  * Date: 31st Octonber 2001
  * Version: 0.51.00
  * Last change:
  *
  **/

package sensei.domains;

import sensei.middleware.gms.GroupHandler;
import sensei.middleware.gms.GroupMember;
import sensei.middleware.gms.Message;
import sensei.middleware.gms.View;
import sensei.middleware.domains.PhaseCoordination;
import sensei.middleware.domains.PhaseCoordinationHolder;
import sensei.middleware.domains.State;
import sensei.middleware.domains.StateTransferType;

import sensei.util.Debug;

/**
  * StateTransferGroupMember without state transfer
  **/
class STGroupMemberStateless extends STGroupMember
{

  //*************************************************************************************//
  //**************************** CONSTRUCTOR ********************************************//
  //*************************************************************************************//

  public STGroupMemberStateless(GroupMember member)
  {
    Trace.code("STGroupMemberStateless.java -> STGroupMemberStateless ( GroupMember member )");
    groupMember = member;
  }

  public GroupMember getGroupMember(){return groupMember;}
  public boolean needsTransfer(){return false;}
  public StateTransferType getStateTransferType(){return StateTransferType.STT_StatelessTransfer;}
  public void freeResources(){groupMember=null;}

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public void processPTPMessage(int sender, Message msg) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void processPTPMessage ( int sender , Message msg )");
    groupMember.processPTPMessage(sender, msg);
  }
  public void processCastMessage(int sender, Message msg) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void processCastMessage ( int sender , Message msg )");
    groupMember.processCastMessage(sender, msg);
  }
  public void memberAccepted(int identity, GroupHandler handler, View theView) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void memberAccepted ( int identity , GroupHandler handler , View theView )");
    groupMember.memberAccepted(identity, handler, theView);
  }
  public void changingView() throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void changingView ( )");
    groupMember.changingView();
  }
  public void installView(View theView) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void installView ( View theView )");
    groupMember.installView(theView);
  }
  public void excludedFromGroup() throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void excludedFromGroup ( )");
    groupMember.excludedFromGroup();
  }

  //*************************************************************************************//
  //**************************** STATE TRANSFER METHODS *********************************//
  //*************************************************************************************//

  public void assumeState() throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void assumeState ( )");
    //it is called, needing transfer or not; do nothing
  }
  public void startTransfer(int[] joiningMembers, PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void startTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder phase )");
    Debug.assert(false, Consts.AREA, "STGroupMemberStateless::startTransfer");
  }
  public State getState(PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> State getState ( PhaseCoordinationHolder phase )");
    Debug.assert(false, Consts.AREA, "STGroupMemberStateless::getState");
    return null;
  }
  public void setState(State s, PhaseCoordination phase) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void setState ( State s , PhaseCoordination phase )");
    Debug.assert(false, Consts.AREA, "STGroupMemberStateless::setState");
  }
  public void stopTransfer(int[] joiningMembers, boolean transferFinished) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void stopTransfer ( int [ ] joiningMembers , boolean transferFinished )");
    Debug.assert(false, Consts.AREA, "STGroupMemberStateless::stopTransfer");
  }
  public void syncTransfer(int coordinator, PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void syncTransfer ( int coordinator , PhaseCoordinationHolder phase )");
    Debug.assert(false, Consts.AREA, "STGroupMemberStateless::syncTransfer");
  }
  public void interruptTransfer(PhaseCoordinationHolder phase) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void interruptTransfer ( PhaseCoordinationHolder phase )");
    Debug.assert(false, Consts.AREA, "STGroupMemberStateless::interruptTransfer");
  }
  public void continueTransfer(int[] joiningMembers, PhaseCoordinationHolder coordinator,
                                        PhaseCoordination joining) throws Exception
  {
    Trace.code("STGroupMemberStateless.java -> void continueTransfer ( int [ ] joiningMembers , PhaseCoordinationHolder coordinator , PhaseCoordination joining )");
    Debug.assert(false, Consts.AREA, "STGroupMemberStateless::continueTransfer");
  }

  //*************************************************************************************//
  //**************************** DATA MEMBERS *******************************************//
  //*************************************************************************************//

  GroupMember groupMember;
}

