/**
  * File: DomainGroupHandlerFactory.java
  * Content: Creates domainGroupHandler instances, to be used in the GMNS
  * Author: LuisM Pena
  * Date: 20th November 2001
  * Version: 0.29.00
  * Last change:
  *
  **/

package sensei.gmns;

import sensei.domains.utils.ExitHandler;

import sensei.middleware.domains.DomainExpulsionReason;
import sensei.middleware.domains.DomainGroupHandler;
import sensei.middleware.domains.DomainGroupUserBaseImpl;
import sensei.middleware.gmns.MapStringSet;

import sensei.util.Error;

/**
  * Creates domainGroupHandler instances, to be used in the GMNS
  **/
class DomainGroupHandlerFactory extends DomainGroupUserBaseImpl
{
  public DomainGroupHandlerFactory(DomainGroupHandler handler, MapStringSet map, GMNSAutoObserver observer,
      ExitHandler exitHandler)
    throws Exception
  {
    Trace.code("DomainGroupHandlerFactory.java -> DomainGroupHandlerFactory ( DomainGroupHandler handler , MapStringSet map , GMNSAutoObserver observer , ExitHandler exitHandler )");
    this.handler=handler;
    this.map=map;
    this.observer=observer;
    this.exitHandler = exitHandler;
  }

  public void domainAccepted(int id)
  {
    Trace.code("DomainGroupHandlerFactory.java -> void domainAccepted ( int id )");
    if (exitHandler!=null)
      exitHandler.acceptedInGroup(handler);
  }
  public void stateObtained(boolean assumed)
  {
    Trace.code("DomainGroupHandlerFactory.java -> void stateObtained ( boolean assumed )");
    new PersistenceHandler(handler, map, assumed, observer);
  }
  public void offendingSubgroup(int id, String reason)
  {
    Trace.code("DomainGroupHandlerFactory.java -> void offendingSubgroup ( int id , String reason )");
    observer.message("Offending subgroup "+id+": "+reason);
  }
  public void domainExpulsed(DomainExpulsionReason reason)
  {
    Trace.code("DomainGroupHandlerFactory.java -> void domainExpulsed ( DomainExpulsionReason reason )");
    if (exitHandler==null)
      observer.excluded();
    else
      exitHandler.excludedFromGroup(reason);
  }
  DomainGroupHandler handler;
  MapStringSet map;
  GMNSAutoObserver observer;
  ExitHandler exitHandler;
}
