/**
  * File: SendMessagesChannelRequester.java
  * Content: Defines a channel requester to send a message
  * Author: LuisM Pena
  * Date: 6th August 2000
  * Version: 0.97.00
  * Last change:
  *				21th August: renamed from SendMessages into CastMessages
  *
  **/

package sensei.gms;

import sensei.middleware.gms.Message;
import sensei.middleware.gms.MessageId;
import sensei.middleware.gms.SenseiGMSMember;

import sensei.util.Debug;

//*************************************************************************************//
//**************************** CAST MESSAGES CHANNEL REQUESTER ************************//
//*************************************************************************************//

/**
  * Channel Requester used to cast messages
  **/
class CastMessagesChannelRequester implements ChannelRequester
{

  public CastMessagesChannelRequester(int thisMember)
  {
    Trace.code("CastMessagesChannelRequester.java -> CastMessagesChannelRequester ( int thisMember )");
    this.thisMember 	= thisMember;
    messages					=	null;
    messageId 				= null;
  }

  public void setMessagesToSend(Message[] messages, MessageId messageId)
  {
    Trace.code("CastMessagesChannelRequester.java -> void setMessagesToSend ( Message [ ] messages , MessageId messageId )");
    this.messages  = messages;
    this.messageId = messageId;
  }

  public boolean invokeMember(CommunicationChannel channel)
  {
    Trace.code("CastMessagesChannelRequester.java -> boolean invokeMember ( CommunicationChannel channel )");
    Debug.assert(messages !=null && messageId!=null && channel!=null,Consts.GMS,"CastMessagesChannelRequester:invokeMember:1");

    boolean ret;

    SenseiGMSMember channelMember = channel.getLink();

    Debug.assert(channelMember!=null,Consts.GMS,"CastMessagesChannelRequester:invokeMember:2");

    try
    {
      ret=channelMember.receiveMessages(thisMember, messages, messageId) && channel.isReliable();
    }
    catch(Exception ex)
    {
      ret=false;
      Trace.handledException(ex);
    }
    return ret;
  }

  int 				thisMember;
  Message[]		messages;
  MessageId		messageId;
}
