/**
  * File: ConfirmMessagesChannelRequester.java
  * Content: Defines a channel requester to confirm a message previously sent
  * Author: LuisM Pena
  * Date: 6th August 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import sensei.middleware.gms.Message;
import sensei.middleware.gms.MessageId;
import sensei.middleware.gms.SenseiGMSMember;

import sensei.util.Debug;

//*************************************************************************************//
//**************************** CONFIRM MESSAGES CHANNEL REQUESTER *********************//
//*************************************************************************************//

/**
  * Channel Requester used to confirm a message previously sent
  **/
class ConfirmMessagesChannelRequester implements ChannelRequester
{

  public ConfirmMessagesChannelRequester(int thisMember)
  {
    Trace.code("ConfirmMessagesChannelRequester.java -> ConfirmMessagesChannelRequester ( int thisMember )");
    this.thisMember 	= thisMember;
    messageId 	= null;
  }

  public void setMessageIdToConfirm(MessageId messageId)
  {
    Trace.code("ConfirmMessagesChannelRequester.java -> void setMessageIdToConfirm ( MessageId messageId )");
    this.messageId = messageId;
  }

  public boolean invokeMember(CommunicationChannel channel)
  {
    Trace.code("ConfirmMessagesChannelRequester.java -> boolean invokeMember ( CommunicationChannel channel )");
    Debug.assert(messageId!=null && channel!=null, Consts.GMS,"ConfirmMessagesChannelRequester:invokeMember:1");

    boolean ret;

    SenseiGMSMember channelMember = channel.getLink();

    Debug.assert(channelMember!=null,Consts.GMS,"ConfirmMessagesChannelRequester:invokeMember:2");

    try
    {
      ret=channelMember.confirmMessages(thisMember, messageId) && channel.isReliable();
    }
    catch(Exception ex)
    {
      ret=false;
      Trace.handledException(ex);
    }
    return ret;
  }

  int 			thisMember;
  MessageId	messageId;
}
