/**
  * File: SendMessagesChannelRequester.java
  * Content: Defines a channel requester to send a message
  * Author: LuisM Pena
  * Date: 21th August 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import sensei.middleware.gms.Message;
import sensei.middleware.gms.MessageId;
import sensei.middleware.gms.SenseiGMSMember;

import sensei.util.Debug;

//*************************************************************************************//
//**************************** SEND MESSAGES CHANNEL REQUESTER ************************//
//*************************************************************************************//

/**
  * Channel Requester used to send a message to some specific member
  **/
class SendMessagesChannelRequester implements ChannelRequester
{

  public SendMessagesChannelRequester(int thisMember)
  {
    Trace.code("SendMessagesChannelRequester.java -> SendMessagesChannelRequester ( int thisMember )");
    this.thisMember 	= thisMember;
    messages 		=	null;
    messageId   = null;
  }

  public void setMessagesToSend(Message[] messages, MessageId messageId)
  {
    Trace.code("SendMessagesChannelRequester.java -> void setMessagesToSend ( Message [ ] messages , MessageId messageId )");
    this.messages  = messages;
    this.messageId = messageId;
  }

  public boolean invokeMember(CommunicationChannel channel)
  {
    Trace.code("SendMessagesChannelRequester.java -> boolean invokeMember ( CommunicationChannel channel )");
    Debug.assert(messages !=null && messageId !=null && channel!=null,Consts.GMS,"SendMessageChannelRequester:invokeMember:1");

    boolean ret;

    SenseiGMSMember channelMember = channel.getLink();

    Debug.assert(channelMember!=null,Consts.GMS,"SendMessageChannelRequester:invokeMember:2");

    try
    {
      ret=channelMember.receivePTPMessages(thisMember, messages, messageId) && channel.isReliable();
    }
    catch(Exception ex)
    {
      ret=false;
      Trace.handledException(ex);
    }
    return ret;
  }

  int					thisMember;
  Message[] 	messages;
  MessageId   messageId;
}
