/**
  * File: WrongMessageCenter.java
  * Content: MessageCenter with wrong behaviour
  * Author: LuisM Pena
  * Date: 7th August 2000
  * Version: 0.97.00
  * Last change:
  *
  *
  **/

package sensei.gms;

import sensei.middleware.gms.InternalView;
import sensei.middleware.gms.InternalViewId;
import sensei.middleware.gms.Message;
import sensei.middleware.gms.MessageId;
import sensei.middleware.gms.SenseiGMSMember;

import java.util.LinkedList;

/**
  * MessageCenter with wrong behaviour
  *			It allows to delay the send of messages until the next view or until the
  *			message flushing
  **/
class WrongMessageCenter  extends MessageCenter
{

//*************************************************************************************//
//**************************** CONSTRUCTOR ********************************************//
//*************************************************************************************//

  /**
    * The MessageCenter must know the SenseiGMSMember that will receive the events
    **/
  public WrongMessageCenter(GroupEventsDispatcher dispatcher, WrongBehaviour behaviour)
  {
    super(dispatcher);
    Trace.code("WrongMessageCenter.java -> WrongMessageCenter ( GroupEventsDispatcher dispatcher , WrongBehaviour behaviour )");

    delayToNextView=false;
    nextMessages=null;
    delayToFlush = false;

    if (behaviour.delaySendOfMessagesToNextView())
    {
      delayToNextView = true;
      nextMessages = new LinkedList();
    }
    if (behaviour.delaySendOfMessagesToFlush())
    {
      delayToFlush = true;
    }
  }

//*************************************************************************************//
//**************************** CAST MESSAGE *******************************************//
//*************************************************************************************//

  public synchronized boolean castMessage(Message message)
  {
    Trace.code("WrongMessageCenter.java -> synchronized boolean castMessage ( Message message )");
    if (delayToNextView)
    {
      nextMessages.add(message);
      return false;
    }
    else
    {
      return super.castMessage(message);
    }
  }

//*************************************************************************************//
//**************************** GET MESSAGES TO SEND ***********************************//
//*************************************************************************************//

  public synchronized boolean hasMessagesToSend()
  {
    Trace.code("WrongMessageCenter.java -> synchronized boolean hasMessagesToSend ( )");
    return delayToFlush && !installingView? false : super.hasMessagesToSend();
  }

//*************************************************************************************//
//**************************** NEW VIEW ***********************************************//
//*************************************************************************************//

  /**
    * Notifies to the message center that a new view is installed.
    * This call won't be back until every message has been processed in
    * the previous view and the view is notified
    **/
  public synchronized boolean newView(InternalView view, int[] leavingMembers)
  {
    Trace.code("WrongMessageCenter.java -> synchronized boolean newView ( InternalView view , int [ ] leavingMembers )");
    boolean ret = super.newView(view, leavingMembers);
    if (ret && delayToNextView)
    {
      LinkedList temp = messagesQueue;
      messagesQueue = nextMessages;
      nextMessages = temp;
    }
    return ret;
  }

//*************************************************************************************//
//**************************** DATA MEMBERS *******************************************//
//*************************************************************************************//

  LinkedList nextMessages;
  boolean delayToNextView, delayToFlush;
}
