/**
  * File: WrongViewHandler.java
  * Content: Wrong version of the ViewHandler
  * Author: LuisM Pena
  * Date: 12th August 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.gms;

import sensei.middleware.gms.InternalView;
import sensei.middleware.gms.Message;
import sensei.middleware.gms.MessageId;
import sensei.middleware.gms.SenseiGMSMember;
import sensei.middleware.gms.Token;

/**
  * Wrong version of the WrongActiveCommunicationChannelsManager
  **/
class WrongViewHandler extends ViewHandler
{

  /**
    * @param messagingBehaviour matching one of the values MESSAGES_xxxx
    **/
  public WrongViewHandler(SenseiGMSMember owner, InternalView initialView,
                                            WrongBehaviour behaviour)
  {
    super (owner, initialView);
    Trace.code("WrongViewHandler.java -> WrongViewHandler ( SenseiGMSMember owner , InternalView initialView , WrongBehaviour behaviour )");

    exitAfterMessages 				= behaviour.exitAfterMessagesSent();
    exitAfterViews						= behaviour.exitAfterViewSent();
    wrongMessageConfirmation	= behaviour.isMessageConfirmationWrong();

    this.behaviour            = behaviour;

    if (behaviour.isMessagesWrong())
      castMessagesChannelRequester		= new WrongCastMessagesChannelRequester(thisMember, behaviour.isMessagesToSentToAllButOne());
    if (behaviour.isViewWrong())
      sendViewChannelRequester				= new WrongSendViewChannelRequester(thisMember, behaviour.isViewToSentToAllButOne());
    if (wrongMessageConfirmation)
      confirmMessagesChannelRequester	= new WrongConfirmMessagesChannelRequester(thisMember);
  }

//*************************************************************************************//
//**************************** CAST MESSAGES ******************************************//
//*************************************************************************************//

  public boolean castMessages(Message[] messages, MessageId messageId)
  {
    Trace.code("WrongViewHandler.java -> boolean castMessages ( Message [ ] messages , MessageId messageId )");
    boolean ret = super.castMessages(messages, messageId);

    if (exitAfterMessages)
      System.exit(0);

    if (!ret && behaviour.isMessagesWrong())
      castMessagesChannelRequester = new WrongCastMessagesChannelRequester(thisMember, behaviour.isMessagesToSentToAllButOne());

    return ret;
  }

//*************************************************************************************//
//**************************** CONFIRM MESSAGES ***************************************//
//*************************************************************************************//

  public boolean confirmMessages(MessageId messageId)
  {
    Trace.code("WrongViewHandler.java -> boolean confirmMessages ( MessageId messageId )");
    boolean ret = super.confirmMessages(messageId);

    if (wrongMessageConfirmation)
      System.exit(0);

    if (!ret && wrongMessageConfirmation)
      confirmMessagesChannelRequester = new WrongConfirmMessagesChannelRequester(thisMember);

    return ret;
  }

//*************************************************************************************//
//**************************** SEND VIEW **********************************************//
//*************************************************************************************//

  public boolean sendView(InternalView view, Token viewToken)
  {
    Trace.code("WrongViewHandler.java -> boolean sendView ( InternalView view , Token viewToken )");
    boolean ret = super.sendView(view, viewToken);

    if (exitAfterViews)
      System.exit(0);

    if (!ret && behaviour.isViewWrong())
      sendViewChannelRequester = new WrongSendViewChannelRequester(thisMember, behaviour.isMessagesToSentToAllButOne());

    return ret;
  }

  boolean exitAfterMessages;
  boolean exitAfterViews;
  boolean wrongMessageConfirmation;
  WrongBehaviour behaviour;
}
