/**
  * File: IntWrapper.java
  * Content: Object wrapper for integers (Integer is not suitable here)
  * Author: LuisM Pena
  * Date: 23th November 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.util;

/**
  * Object wrapper for integers (Integer is not suitable here)
  * The reason because is not suitable is because Integer does not allow to
  * change the contained integer. Therefore, when a table is indexed with integers
  * (wrapped to java.lang.Integer), any search on the table must be performed using
  * new objects, when the intention through the design of sensei has been to
  * minimize the creation of objects.
  * A probably reason justifying the design of the Integer class, not allowing the
  * change of the integer, is the hashCode function. As stated in the Object
  * documentation, the hashCode should return consistently the same value during the
  * whole life cicle of the object. This can not be assured with IntWrapper, and
  * therefore, it is <b>very important</b> that an IntWrapper used in a table will
  * never change its value. Inside Sensei, this is a fact, but if this class is used
  * outside Sensei, the user must be aware of the previous restriction!!!
  **/
public class IntWrapper implements Comparable, Cloneable
{
  public int i;

  public IntWrapper(){i=0;}
  public IntWrapper(int integer){i=integer;}
  public int hashCode(){return i;}
  public boolean equals(Object o) {return o instanceof IntWrapper? ((IntWrapper)o).i==i : false;}
  public int compareTo(Object o){return i-((IntWrapper)o).i;}
  public String toString(){return String.valueOf(i);}
}
