/**
  * File: ShutdownHook.java
  * Content: Defines a class that registers ShutdownHooker instances
  *						and inform them when the Java VM is shutting down
  * Author: LuisM Pena
  * Date: 4th November 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.util;

import java.util.HashSet;
import java.util.Iterator;

/**
  * ShutdownHook registers ShutdownHooker instances and inform them when the Java
  * VM is shutting down
  **/
public class ShutdownHook
{
  /**
    * Registers a new shutdown hooker instance, if not registered previously
    **/
  static public void register(ShutdownHooker hooker)
  {
//		Not possible to debug at this level yet. And not needed, by the way
//		Debug.assert(hooker!=null, "ShutdownHook", "Registering a null instance");
    synchronized (hookers)
    {
      hookers.add(hooker);
    }
  }

  /**
    * Unregisters a shutdown hooker instance, if previously registered
    **/
  static public void unregister(ShutdownHooker hooker)
  {
    synchronized (hookers)
    {
      hookers.remove(hooker);
    };
  }

  /**
    * This method is called when the Java VM is shutting down.
    **/
  static public void shuttingDown()
  {
    synchronized (hookers)
    {
      Iterator iterator = hookers.iterator();
      while (iterator.hasNext())
        ((ShutdownHooker)(iterator.next())).shuttingDown();
    }
  }

  static HashSet hookers = new HashSet();
  static
  {
    //it's needed to verify that the file will be close at the end!
    Runtime.getRuntime().addShutdownHook(new Thread(){public void run(){ShutdownHook.shuttingDown();}});
  }
}
