/**
  * File: TraceInfo.java
  * Content: Retrieves information about an exception
  * Author: LuisM Pena
  * Date: 10th February 2002
  * Version: 0.97.01
  * Last change:
  *
  **/

package sensei.util;

import java.io.*;
import java.util.*;
/**
  * Parses the trace information on an exception
  **/
public class TraceInfo extends Thread
{
  public TraceInfo(Throwable ex)
  {
    parse(ex);
    if (!error)
    {
      exception=(String) trace.remove(0);
    }
  }

  public TraceInfo()
  {
    Exception ex = new Exception();
    ex.fillInStackTrace();
    parse(ex);
    if (!error)
    {
      exception=(String) trace.remove(0);
      trace.remove(0); //it does not mind the first element!
    }
  }

  public String getException()
  {
    return exception;
  }

  public int getStackSize()
  {
    return trace.size();
  }

  /**
   * Exception IndexOutOfBoundException can be thrown
   */
  public String getStackLine(int n)
  {
    return transform((String) trace.get(n));
  }

  public String getStackOrigin()
  {
    return getStackLine(getStackSize()-1);
  }

  public String getStackEnd()
  {
    return getStackLine(0);
  }

  private String transform(String line)
  {
    return line.trim().substring(3);
  }

  public String toString()
  {
    StringBuffer ret = new StringBuffer();
    Iterator it = trace.iterator();
    boolean first = true;
    while(it.hasNext())
    {
      if (first)
        first=false;
      else
        ret.append(" - ");
      ret.append(transform((String) it.next()));
    }
    return ret.toString();
  }

  void parse(Throwable ex)
  {
    trace = new ArrayList();
    try
    {
      PipedWriter pipe = new PipedWriter();
      PrintWriter pw = new PrintWriter(pipe);
      reader = new PipedReader(pipe);
      start();
      ex.printStackTrace(pw);
      pw.close();
      try{join();}catch(InterruptedException iex){}
    }
    catch(Exception exx){}
  }

  public void run()
  {
    error=false;
    try
    {
      BufferedReader br = new BufferedReader(reader);
      String line=br.readLine();
      while(line!=null)
      {
        trace.add(line);
        line = br.readLine();
      }
    }
    catch(IOException ex){error=true;}
  }

  String exception;
  boolean error;
  Reader reader;
  List trace;
}
