/**
  * File: MemoryLogStream.java
  * Content: Log stream storing all the logging in memory
  * Author: LuisM Pena
  * Date: 7th November 2000
  * Version: 0.97.00
  * Last change:
  *
  **/

package sensei.util.logging;

import sensei.util.ShutdownHook;
import sensei.util.ShutdownHooker;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

class MemoryLogStream implements LogStream, ShutdownHooker
{

  /*******************************************************************************/
  /**************************** CONSTRUCTOR **************************************/
  /*******************************************************************************/

  public MemoryLogStream()
  {
    log=new ArrayList(50);
    name=null;
    ShutdownHook.register(this);
  }

  /*******************************************************************************/
  /**************************** SHUTDOWN HOOKER INTERFACE ************************/
  /*******************************************************************************/

  public synchronized void shuttingDown()
  {
    if (!log.isEmpty())
      for (Iterator it=log.iterator(); it.hasNext();)
        System.out.println((String) it.next());
    log.clear();
  }

  /*******************************************************************************/
  /**************************** DUMP TO LOG STREAM *******************************/
  /*******************************************************************************/

  public synchronized void dumpToLogStream(LogStream stream)
  {
    if (!log.isEmpty())
      for (Iterator it=log.iterator(); it.hasNext();)
        stream.println((String) it.next());
    if (name!=null)
      stream.setLogName(name);
    log.clear();
  }

  /*******************************************************************************/
  /**************************** LOG STREAM INTERFACE *****************************/
  /*******************************************************************************/

  public void setLogName(String name)
  {
    this.name=new String(name);
  }

  public synchronized void println(String line)
  {
    log.add(line);
  }

  public synchronized void close()
  {
    log.clear();
    name=null;
    ShutdownHook.unregister(this);
  }

  List log;
  String name;
}
